@@ -1,6 +1,6 @@
-from core.actors import ParallelComputeActor
-from core.ports.inputports import TextInputPort
-from core.ports.outputports import TextOutputPort
+from cloudam_stream.core.actors import ParallelComputeActor
+from cloudam_stream.core.ports.inputports import TextInputPort
+from cloudam_stream.core.ports.outputports import TextOutputPort
 
 
 class MyParallelComputeActor(ParallelComputeActor):
@@ -13,5 +13,11 @@ class MyParallelComputeActor(ParallelComputeActor):
         "parallelism": 2,
     }
 
+    ss = ""
+
+    def begin(self):
+        self.sss = ""
+
     def process(self, item, port):
+
         self.success.emit(item)
