@@ -11,7 +11,7 @@ class MySinkActor(SinkActor):
     intake = TextInputPort()
 
     def begin(self):
-        self.file = open("result.txt", "w+")
+        self.file = open("/home/cloudam/result.txt", "w+")
         print("------start sink actor")
         self.i = 0
 
